﻿
//-----------------------------------------------------------------------
// <copyright file="Program.cs" company="FRMSc Ltd.">
//     Copyright © FRMSc Ltd. All rights reserved.
// </copyright>
//---

using Newtonsoft.Json;
using System;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Runtime.InteropServices;
using System.Threading;

namespace Safe.API.SampleClient
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                #region Initialise adapter passing base safe URL and user account credentials.

                var webApiUrl = @"https://web-euwest.frmsc.com";
                var identityUrl = @"https://identity-eu.frmsc.com";
                var clientId = "[Your_ClientId]";
                var clientSecret = "[your_ClientSecret]";
                var apiKey = "[Your_ApKey]";

                var adapter = new TokenAuthenticationAdapter(webApiUrl, identityUrl, clientId, clientSecret);

                #endregion

                #region  Submit roster for processing.

                Console.WriteLine("1) Submitting data and acquiring token.");

                var rosterFriendlyName = "Sample roster API post";
                var rosterCsvBlob = File.ReadAllText("SampleRosters\\SAFE_Export.csv");

                var token = adapter.UploadRoster(
                            rosterFriendlyName,
                            rosterCsvBlob,
                            apiKey,
                            false,
                            "" /* optional roster categories as comma delimited string */);


                Console.WriteLine("\tAcquired token: {0}", token);

                #endregion
                
                #region Check status.

                Console.WriteLine("2) Checking status of processing.");
                Console.WriteLine(adapter.GetResults(token, DataKey.Status, apiKey));
                while (adapter.GetResults(token, DataKey.Status, apiKey) == "processing")
                {
                    Console.WriteLine("\tRoster still being processed....");
                    Thread.Sleep(2000);
                }

                #endregion
                
                #region Get results.

                Console.WriteLine("3) Downloading results.");
                
                // Get the status of processing.
                var status = adapter.GetResults(token, DataKey.Status, apiKey);
                Console.WriteLine("\tStatus: {0}", status);

                // Get any feedback.
                var feedback = adapter.GetResults(token, DataKey.Feedback, apiKey);
                Console.WriteLine("\tFeedback: {0}", feedback);

                var schedules = adapter.GetResults(token, DataKey.TableOfScheduleDetailsIds, apiKey);
                Console.WriteLine("\tSchedules: {0}", schedules);

                // Get fatique information in the 100pt scale.
                Console.WriteLine("\tDownloading 100pt scale alertness data.");
                var hundredpointalertness = adapter.GetResults(token, DataKey.HundredPointAlertness, apiKey);                
                File.WriteAllText("Output\\hundredPointAlertnessData.csv", hundredpointalertness);

                // Get fatique information in the Samn Perelli scale.
                Console.WriteLine("\tDownloading Samn Perelli scale alertness data.");
                var samnperellialertness = adapter.GetResults(token, DataKey.SamnPerelliAlertness, apiKey);                
                File.WriteAllText("Output\\samnPerelliAlertnessData.csv", samnperellialertness);

                // Get fatique information in the Karolinska scale.                
                Console.WriteLine("\tDownloading Karolinska scale alertness data.");
                var karolinskaalertness = adapter.GetResults(token, DataKey.KarolinskaAlertness, apiKey);
                File.WriteAllText("Output\\karolinskAlertnessData.csv", karolinskaalertness);

                // Get fatique information in the Karolinska Probability scale.                
                Console.WriteLine("\tDownloading Karolinska Probability alertness data.");
                var karolinskaprobability = adapter.GetResults(token, DataKey.KarolinskaProbability, apiKey);
                File.WriteAllText("Output\\karolinskProbabilityAlertnessData.csv", karolinskaprobability);

                // Get fatique information in the Visgilance Degradation scale.                
                Console.WriteLine("\tDownloading Vigilance Degradation alertness data.");
                var vigilancedegradation = adapter.GetResults(token, DataKey.VigilanceDegradation, apiKey);
                File.WriteAllText("Output\\VigilanceDegradationData.csv", vigilancedegradation);

                // Get fatique information in the Percentage Missed Responses scale.                
                Console.WriteLine("\tDownloading Percentage Missed Responses alertness data.");
                var percentagemissedresponses = adapter.GetResults(token, DataKey.PercentageMissedResponses, apiKey);
                File.WriteAllText("Output\\PercentageMissedResponsesData.csv", percentagemissedresponses);

                // Get fatique information in the Complex Reaction Degradation scale.                
                Console.WriteLine("\tDownloading Complex Reaction Degradation alertness data.");
                var complexreactiondegradation = adapter.GetResults(token, DataKey.ComplexReactionDegradation, apiKey);
                File.WriteAllText("Output\\ComplexReactionDegradationData.csv", complexreactiondegradation);

                // Get the sleep period summary information.                
                Console.WriteLine("\tDownloading sleep periods data.");
                var sleepperiods = adapter.GetResults(token, DataKey.SleepPeriods, apiKey);
                File.WriteAllText("Output\\sleepPeriodData.csv", sleepperiods);

                // Get the updated schedule.                
                Console.WriteLine("\tDownloading schedule data.");
                var schedule = adapter.GetResults(token, DataKey.Schedule, apiKey);
                File.WriteAllText("Output\\scheduleData.csv", schedule);

                // Get a summary of the time spent above threshold                
                Console.WriteLine("\tDownloading time above threshold data.");
                var timeabovethreshold = adapter.GetResults(token, DataKey.TimeAboveThreshold, apiKey);
                File.WriteAllText("Output\\timeAboveThresholdData.csv", timeabovethreshold);
                
                #endregion    
            }
            catch (WebException ex)
            {
                Console.WriteLine(ex.ToString());
            }

            Console.WriteLine("Done..!");
            Console.ReadKey();
       }

     
    }
}
