﻿////-----------------------------------------------------------------------
//// <copyright file="Adapter.cs" company="CM Group Ltd.">
////     Copyright © CM Group Ltd. All rights reserved.
//// </copyright>
////---

//using System;
//using System.IO;
//using System.Net;
//using System.Text;

//namespace Safe.API
//{
//    /// <summary>
//    /// Represents the <see cref="Safe.API.FormsAuthenticationAdapter"/> class in the object model.
//    /// </summary>
//    public class FormsAuthenticationAdapter : IAdapter
//    {
//        private static readonly string POST = "POST";
//        private static readonly string GET = "GET";
//        private static readonly string CONTENTTYPE = "application/x-www-form-urlencoded";
//        private static readonly string AUTHCOOKIE = ".ASPXAUTH";
//        private static readonly string FRIENDLYNAMECOOKIE = "FriendlyName";
//        private static readonly string LOGONURL = "Security/Account/LogOn";
//        private static readonly string LOGONBODYFORMAT = "username={0}&password={1}";
//        private static readonly string UPLOADROSTERURL = "API/UploadRoster";
//        private static readonly string UPLOADROSTERBODYFORMAT = "rosterName={0}&rosterData={1}&overwrite={2}&dateFormat={3}&rosterCategories={4}";
//        private static readonly string GETRESULTSURL = "API/GetResults";
//        private static readonly string GETRESULTSBODYFORMAT = "token={0}&dataKey={1}";
//        private static readonly string REPROCESSALLSCHEDULESURL = "API/ReprocessAllSchedules";

//        private Uri _baseUrl;
//        private string _username;
//        private string _password;
//        private CookieContainer _authenticationCookies;

//        /// <summary>
//        /// Initializes a new instance of the <see cref="Safe.API.FormsAuthenticationAdapter"/> class. 
//        /// </summary>
//        /// <param name="baseApiUrl">The URL to a Safe site.</param>
//        /// <param name="username">The username used to authenticate the client.</param>
//        /// <param name="password">The password used to authenticate the client.</param>
//        public FormsAuthenticationAdapter(string baseApiUrl, string username, string password)
//        {
//            if (string.IsNullOrWhiteSpace(baseApiUrl)) throw new NullReferenceException("baseApiUrl");
//            if (string.IsNullOrWhiteSpace(username)) throw new NullReferenceException("username");
//            if (string.IsNullOrWhiteSpace(password)) throw new NullReferenceException("password");

//            this._baseUrl = new Uri(baseApiUrl);
//            this._username = username;
//            this._password = password;
//            this._authenticationCookies = new CookieContainer();
//        }

//        #region IAdapter members

//        /// <summary>
//        /// Gets the authentication cookie.
//        /// </summary>
//        public Cookie AuthCookie
//        {
//            get
//            {
//                return this.GetCookie(AUTHCOOKIE);
//            }
//        }

//        /// <summary>
//        /// Gets a value indicating whether the adapter is authenticated.
//        /// </summary>
//        public bool IsAuthenticated
//        {
//            get
//            {
//                return (this.AuthCookie != null &&
//                    !this.AuthCookie.Expired);
//            }

//        }

//        /// <summary>
//        /// Uploads a roster to the Safe site, returning a unique token which you can use to check 
//        /// the status of processing and gain access to the results.
//        /// </summary>
//        /// <param name="rosterName">A <see cref="System.String"/> object representing the friendly name for a roster. This will appear in the Safe UI.</param>
//        /// <param name="rosterData">A <see cref="System.String"/> object representing a roster CSV blob.</param>
//        /// <param name="rosterData">A <see cref="System.Boolean"/> object indicating whether to overwrite any existing rosters with the same name.</param>
//        /// <param name="dateFormat">A <see cref="Safe.API.SupportedDateFormat"/> object indicating the date format in which to parse the dates in the roster.</param>
//        /// <returns>A <see cref="System.String"/> object.</returns>
//        public string UploadRoster(string rosterName, string rosterData, bool overwrite, SupportedDateFormat dateFormat, string rosterCategories = null)
//        {
//            if (!this.IsAuthenticated)
//                this.Authenticate();

//            var request = this.BuildRequest(
//                UPLOADROSTERURL,
//                string.Format(
//                    UPLOADROSTERBODYFORMAT,
//                    rosterName,
//                    rosterData,
//                    overwrite,
//                    dateFormat == SupportedDateFormat.systemSetting ?
//                        null :
//                        dateFormat.ToString(),
//                    rosterCategories));

//            return this.ReadDataFromResponse(request);
//        }

//        /// <summary>
//        /// Gets the processed data for the roster associated with a given token. Possible return values are:
//        /// <para>404 - Token not recognised</para> 
//        /// <para>403 - Roster is currently being processed or it completed with an error.</para> 
//        /// <para>500 - Unexpected error. Could not find processed roster</para> 
//        /// <para>204 - Index value not supported</para> 
//        /// <para>A <see cref="System.String"/> object representing the processing status (processing, completedsuccessfully, or completedwitherror)</para> 
//        /// <para>A <see cref="System.String"/> object containing any feedback from the processing. This will include any error data.</para> 
//        /// <para>A <see cref="System.String"/> object containing a CSV blob representing the processed data.</para> 
//        /// </summary>
//        /// <param name="token">A <see cref="System.String"/> object representing the unique token.</param>
//        /// <param name="dataKey">A <see cref="Safe.API.DataKey"/> object representing the data you want to retrieve.</param>
//        /// <returns>A <see cref="System.String"/> object.</returns>
//        public string GetResults(string token, DataKey key)
//        {
//            if (!this.IsAuthenticated)
//                this.Authenticate();

//            var request = this.BuildRequest(
//                GETRESULTSURL,
//                string.Format(
//                    GETRESULTSBODYFORMAT,
//                    token,
//                    key.ToString()));

//            return this.ReadDataFromResponse(request);
//        }

//        /// <summary>
//        /// Gets the absolute launch URL.
//        /// </summary>
//        /// <param name="token">A <see cref="System.String"/> object representing the unique token.</param>
//        /// <returns>A <see cref="System.String"/> object containing the absolute URL</returns>
//        public string GetLaunchUrl(string token)
//        {
//            return string.Format(
//                    "{0}API/Launch?token={1}",
//                    this._baseUrl.AbsoluteUri,
//                    token);
//        }

//        #endregion

//        private HttpWebRequest BuildRequest(string actionRelativeUrl, string postBody = null)
//        {
//            var absoluteUrl = new Uri(this._baseUrl, actionRelativeUrl);

//            var request = (HttpWebRequest)WebRequest.Create(absoluteUrl);

//            if (!string.IsNullOrEmpty(postBody))
//            {
//                request.Method = POST;
//                request.ContentType = CONTENTTYPE;
//                request.CookieContainer = this._authenticationCookies;

//                using (var stream = request.GetRequestStream())
//                {
//                    stream.Write(Encoding.UTF8.GetBytes(postBody), 0, postBody.Length);
//                }
//            }
//            else
//            {
//                request.Method = GET;
//            }

//            return request;
//        }

//        private string ReadDataFromResponse(HttpWebRequest request)
//        {
//            var response = request.GetResponse() as HttpWebResponse;

//            if (response == null || response.ContentLength == 0)
//                throw new NotSupportedException("No response returned.");

//            using (var data = new StreamReader(response.GetResponseStream()))
//            {
//                return data.ReadToEnd();
//            }
//        }

//        private void Authenticate()
//        {
//            this._authenticationCookies = new CookieContainer();

//            var request = this.BuildRequest(
//                LOGONURL,
//                string.Format(
//                    LOGONBODYFORMAT,
//                    this._username,
//                    this._password));

//            request.CookieContainer = this._authenticationCookies;
//            request.GetResponse();

//            if (!this.IsAuthenticated)
//                throw new NotSupportedException("Could not authenticate client with provided credentials.");

//        }

//        private Cookie GetCookie(string cookieName)
//        {
//            var cookies = this._authenticationCookies.GetCookies(this._baseUrl);

//            foreach (var cookie in cookies)
//            {
//                var parsedCookie = cookie as Cookie;

//                if (!parsedCookie.Name.Equals(cookieName, StringComparison.OrdinalIgnoreCase))
//                    continue;

//                return parsedCookie;
//            }

//            return null;
//        }
//    }
//}
